import { APIGatewayEvent, SNSEvent } from 'aws-lambda';
import {
  APIGatewayWebSocketEvent,
  createSNSEventProcessor,
  createHttpHandler,
  createWsHandler,
  MemoryConnectionManager,
  MemoryEventStore,
  MemorySubscriptionManager,
  PubSub,
  withFilter,
} from 'aws-lambda-graphql';
import { mergeSchemas } from './mergeSchemas';

const eventStore = new MemoryEventStore();

const pubSub = new PubSub({
  eventStore,
  topic: process.env.snsArn,
});

function createServer(params) {
  const schema = params.schema;
  const connectionManager =
    params.connectionManager || new MemoryConnectionManager();
  const subscriptionManager =
    params.subscriptionManager || new MemorySubscriptionManager();

  const eventProcessor = createSNSEventProcessor({
    connectionManager,
    schema,
    subscriptionManager,
  });
  const wsHandler = createWsHandler({
    connectionManager,
    schema,
    subscriptionManager,
  });
  const httpHandler = createHttpHandler({
    connectionManager,
    schema,
    WSS_URL: process.env.WSS_URL,
  });

  return async function handler(
    event: APIGatewayEvent | APIGatewayWebSocketEvent | SNSEvent,
    context,
  ) {
    context = Object.assign(context, params.context);

    console.log('▶️ received event', JSON.stringify(event, null, '  '));
    if ((event as SNSEvent).Records != null) {
      // event is DynamoDB stream event
      console.log('🚁 SNS Event');
      return eventProcessor(event as SNSEvent, context, null);
    } else if (
      (event as APIGatewayWebSocketEvent).requestContext != null &&
      (event as APIGatewayWebSocketEvent).requestContext.routeKey != null
    ) {
      // event is web socket event from api gateway v2
      console.log('🏎 Websocket Event');
      return wsHandler(event as APIGatewayWebSocketEvent, context);
    } else if (
      (event as APIGatewayEvent).requestContext != null &&
      (event as APIGatewayEvent).requestContext.path != null
    ) {
      // event is http event from api gateway v1
      console.log('☎️ HTTP Event');
      return httpHandler(event as APIGatewayEvent, context, null as any);
    } else {
      throw new Error('Invalid event');
    }
  };
}

module.exports = {
  createServer,
  mergeSchemas,
};
